@@ -3,7 +3,7 @@
 #' RFc: R client for FetchClimate service
 #'
 #' Extracts raw, averaged environmental data (such as air temperature, precipitation rate, wind speed,
-#' etc.) published at the FetchClimate Web service (<http://fc.itis.cs.msu.ru/>) for the specified geo-locations and time bounds from
+#' etc.) published at the FetchClimate Web service (<http://fetchclimate.org/>) for the specified geo-locations and time bounds from
 #' different data sets.
 #' 
 #' @section Time series fetching functions:
@@ -11,24 +11,90 @@
 #'
 #' @section Gridded data fetching functions:
 #' fcGrid
+#' 
+#' @section Listing available data functions:
+#' fcDataSets fcVariables
 #'
 #' @docType package
 #' @name RFc
 #' 
 NULL
 
+
+#' Availabe data
+#' 
+#' The default instance of FetchClimate at <http://fetchclimate2.cloudapp.net/> provides the following data
+#' 
+#' \strong{Environmental variables:}\cr
+#' "abshum" : Absolute air humidity (g/m^3)\cr
+#' "airt" : Air temperature near surface (Degrees C)\cr
+#' "airt_land": Air temperature near surface. Land only areas (Degrees C)\cr
+#' "airt_ocean" : Air temperature near surface. Ocean only areas (Degrees C)\cr
+#' "depth_ocean" : Depth below sea level. Ocean only areas (meters)\cr
+#' "dtr" : Diurnal air temperature rate (Degrees C)\cr
+#' "elev" : Elevation above sea level (meters)\cr
+#' "elev_land" : Elevation above sea level. Land only a areas (meters)\cr
+#' "frs" : Frost days frequency (days/month)\cr
+#' "pet": Potential evapotranspiration (mm/month)\cr
+#' "prate" : Precipitation rate ()\cr
+#' "relhum" : Relative humidity (percentage)\cr
+#' "relhum_land" : Relative humidity. Land only area (percentage)\cr
+#' "soilmoist" : Soil moisture (mm/m)\cr
+#' "sunp" : Sunshine fraction (Percent of maximum possible sunshine)\cr
+#' "wet" : Wet days frequency (days/month)\cr
+#' "windspeed" : Wind speed at 10m (m/s)\cr
+#' "wvp" : Water vapour pressure (hPa)\cr
+#' "wvsp" : Water vapour saturation pressure (hPa)\cr
+#' 
+#' \strong{DataSets [supported variables]:}\cr
+#' "CESM1-BGC airt" [airt]\cr
+#' "CESM1-BGC prate" [prate]\cr
+#' "CpcSoilMoisture" [soilmoist]\cr
+#' "CRU CL 2.0" [airt, airt_land, dtr, frs, prate, relhum, relhum_land, sunp, wet, windspeed]\cr
+#' "ETOPO1" [elev]\cr
+#' "FC1 Variables" [airt_land, airt_ocean, depth_ocean, relhum_land]\cr
+#' "GTOPO30" [elev, elev_land]\cr
+#' "Malmstrom PET" [pet]\cr
+#' "NCEP/NCAR Reanalysis 1 (Gauss T62)" [prate]\cr
+#' "NCEP/NCAR Reanalysis 1 (regular grid)" [airt]\cr
+#' "WagnerWVSP" [abshum, wvp, wvsp]\cr
+#' "WorldClim 1.4" [airt, airt_land, prate]\cr
+#' 
+#' Note that availavle variables as well as datasets are subject to change in time
+#' The list above describes the most recent configuration
+#' However specifing timestamp parameter other than "Now" may lead to different set of available data
+#' Verify the actual list using `fcVariables()` and `fcDataSets()` functions
+#' 
+#' @name Availabe data
+
+NULL
+
+#' Release notes
+#' 
+#' \strong{release 0.1-2}\cr
+#' Support for obtaining available datasets and varaibles with\cr
+#'      fcDataSets()\cr
+#'      fcVaraibles()\cr
+#'      \cr
+#' \strong{release 0.1-1}\cr
+#' Support for fetching simple time series and grids
+#' 
+#' @name Release notes
+
 require(jsonlite)
 require(sp)
 require(httr)
 
-internal_fc.formRequestBody <- function(envVar, # must be private
+
+internal_fc.formRequestBody <- function(envVar,
                                         lat,lon,
                                         years,
                                         days,
                                         hours,
-                                        spatialRegionType,dataSets,timestampStr) {
+                                        spatialRegionType,dataSets,timestampStr,
+                                        verbose) {
         ## JSON request object. 
-        ## The text may be downloaded from http://fc.itis.cs.msu.ru/form
+        ## The text may be downloaded from http://fetchclimate2.cloudapp.net/form
         
         timeRegion <- list(
                 Years=years,
@@ -83,7 +149,11 @@ internal_fc.formRequestBody <- function(envVar, # must be private
                 ReproducibilityTimestamp=unbox(timestamp)
         )
         j <- toJSON(request,digits=20, pretty=TRUE)
-        #print(j)
+        if(verbose) {
+                print("request JSON:");
+                print(j)        
+        }
+        
         return(j)
 }
 
@@ -187,7 +257,7 @@ internal_fc.TimeSeries <-function(envVar, #must be private
                                   years,
                                   days,
                                   hours,
-                                  url,dataSets,timestampStr) {
+                                  url,dataSets,timestampStr,verbose) {
         N <- length(lat)
         if(length(lon) != N) {
                 stop("lon and lat must be the same length");
@@ -199,10 +269,11 @@ internal_fc.TimeSeries <-function(envVar, #must be private
                                             hours,
                                             spatialRegionType="Points",
                                             dataSets=dataSets,
-                                            timestampStr=timestampStr);
+                                            timestampStr=timestampStr,
+                                            verbose=verbose);
         requestProvenance <- length(dataSets)>1 || dataSets=="ANY"  
         
-        result <- internal_fc.fetchCore(json,url,requestProvenance)    
+        result <- internal_fc.fetchCore(json,url,requestProvenance,verbose)    
         conf <- internal_fc.getConfiguration(url,timestampStr)
         explicitDs <- c()
         if(requestProvenance){
@@ -217,12 +288,14 @@ internal_fc.TimeSeries <-function(envVar, #must be private
         return(resultMatrix)
 }
 
-internal_fc.fetchCore <- function(jsonRequest,url,requestProvenance) {
+internal_fc.fetchCore <- function(jsonRequest,url,requestProvenance,verbose) {
         #print("requesting JSON")
         #print(jsonRequest)
         replyRaw <- POST(url,path="/api/compute",accept("text/plain"),content_type("application/json"),body=jsonRequest)
         reply<-content(replyRaw)
-        print(reply)
+        if(verbose) {
+                print(reply)
+        }        
         
         ## wait while processing in progress
         while (substr(reply,1,7)=='pending' || substr(reply,1,8)=='progress') {
@@ -231,15 +304,20 @@ internal_fc.fetchCore <- function(jsonRequest,url,requestProvenance) {
                 replyRaw <- GET(url,
                                 path="/api/status",
                                 query=list(hash=hash),
+                                content_type("application/json"),
                                 accept("text/plain")) 
                 reply = content(replyRaw)
-                print(reply)
+                if(verbose) {
+                        print(reply)
+                }
         }
         
         result <- c()
         
         ## get result data
-        print("Receiving data...")
+        if(verbose) {
+                print("Receiving data...")
+        }
         if (substr(reply,1,9)=='completed') {
                 msds = substr(reply,11,nchar(reply))    
                 vars_to_fetch <- c()
@@ -261,6 +339,81 @@ internal_fc.fetchCore <- function(jsonRequest,url,requestProvenance) {
         return(result);
 }
 
+#' Fetches information about available datasets
+#' @import httr
+#' @import jsonlite
+#' @import sp
+#' @export
+#' @param url The URL of the service to get the information about
+#' @param timestamp A character scalar. A string containing the time for which the result must correspond. The format is "YYYY-MM-DD". The special value "NOW" fetch the data using the latest FetchClimate configuration available.
+#' @return Type: list\cr
+#' Contains a descriptions of the datasets available to fetch the data from
+#' 
+#' @examples
+#' #Getting currently available datasets
+#' 
+#' fcDataSets()
+#' 
+#' #Getting datasets available on 1 June 2016
+#' 
+#' fcDataSets(timestamp='2016-06-01')
+#' 
+#' #Listing varaibles that are avaialable from "CRU CL 2.0" dataset
+#' 
+#' fcDataSets(timestamp='2016-06-01')$`CRU CL 2.0`$Variables
+#' 
+fcDataSets <- function(url='http://fetchclimate2.cloudapp.net/',timestamp="NOW") {
+        configuration <- internal_fc.getConfiguration(url,timestamp)
+        result <- list()
+        for(i in 1:length(configuration$DataSources)) {
+                currentDs <- configuration$DataSources[[i]]
+                dataset <- list(
+                        Name=currentDs$Name,
+                        Description=currentDs$Description,
+                        Copyright=currentDs$Copyright,
+                        Variables=as.character(currentDs$ProvidedVariables))
+                
+                result[[currentDs$Name]] <- dataset
+        }
+        return(result)
+}
+
+#' Fetches information about available variables
+#' @import httr
+#' @import jsonlite
+#' @import sp
+#' @inheritParams fcDataSets
+#' @export
+#' @return Type: list\cr
+#' Contains a descriptions of the variables available for fetching
+#' 
+#' @examples
+#' #Getting currently available variables
+#' 
+#' fcVariables()
+#' 
+#' #Getting variables available on 1 June 2016
+#' 
+#' fcVariables(timestamp='2016-06-01')
+#' 
+#' #Obtaining units of "prate" variable
+#' 
+#' fcVariables(timestamp='2016-06-01')$`prate`$Units
+#' 
+fcVariables <- function(url='http://fetchclimate2.cloudapp.net/',timestamp="NOW") {
+        configuration <- internal_fc.getConfiguration(url,timestamp)
+        result <- list()
+        for(i in 1:length(configuration$EnvironmentalVariables)) {
+                currentVar <- configuration$EnvironmentalVariables[[i]]
+                v <- list(
+                        Name=currentVar$Name,
+                        Description=currentVar$Description,
+                        Units=currentVar$Units)
+                
+                result[[currentVar$Name]] <- v
+        }
+        return(result)
+}
 
 #' Fetches time series data for a set of locations
 #' 
@@ -268,19 +421,20 @@ internal_fc.fetchCore <- function(jsonRequest,url,requestProvenance) {
 #' 
 #' 
 #' @name TimeSeries
-#' @param variable An identifier of the variable to fetch.
-#' To see the full list of supported variables navigate to the service url with the browser (e.g. http://fc.itis.cs.msu.ru), explore the "What?" tab in the web application for available variables list.
+#' @param variable An identifier of the variable to fetch. \cr
+#' To get the list of available variables use `fcVariables()` function or see "Available data" section of documentaion
 #' @param latitude A numeric vector. Latitudes of the point set to fetch values for
 #' @param longitude A numeric vector. Longitudes of the point set to fetch values for
 #' @param firstYear A numeric scalar. Temporal coverage definition: The lower bound of years over which the averaging is performed
 #' @param lastYear A numeric scalar. Temporal coverage definition: The upper bound of years over which the averaging is performed
 #' @param firstDay A numeric scalar. Temporal coverage definition: The lower bound of the days interval within each year over which the averaging is performed
 #' @param lastDay A numeric scalar. Temporal coverage definition: The upper bound of the days interval within each year over which the averaging is performed
-#' @param startHour A numeric scalar. Temporal coverage definition: The lower bound of the days interval within each year over which the averaging is performed
-#' @param stopHour A numeric scalar. Temporal coverage definition: The upper bound of the days interval within each year over which the averaging is performed
+#' @param startHour A numeric scalar. Temporal coverage definition: The lower bound of the hours interval within each day over which the averaging is performed
+#' @param stopHour A numeric scalar. Temporal coverage definition: The upper bound of the hours interval within each day over which the averaging is performed
 #' @param url The URL of the service to query the data from
 #' @param dataSets A character vector. An identifier of the data set to fetch the data from. The special value "ANY" enables data stitching from all available data sets.
 #' @param reproduceFor A character scalar. A string containing the time for which the result must correspond. The format is "YYYY-MM-DD". The special value "NOW" fetch the data using the latest FetchClimate configuration available.
+#' @param verbose A logical scalar. If set to TRUE the method outputs its actions verbosely
 #' @return A list. Contains the following entries: values, sd, provenance.
 #' Each of entries have the following dimensions (using values as an example):
 #' length(values) = point set count * time series length;
@@ -297,7 +451,7 @@ internal_fc.fetchCore <- function(jsonRequest,url,requestProvenance) {
 #' fcTimeSeriesHourly(variable="airt",latitude=55.5, longitude=37.3,
 #'      firstDay=183,lastDay=213,
 #'      firstYear=2008,lastYear=2008,
-#'      startHour=0,stopHour=24)
+#'      startHour=0,stopHour=23)
 #' 
 NULL
 
@@ -311,10 +465,11 @@ fcTimeSeriesYearly<-function(
         latitude,longitude,
         firstYear,lastYear,
         firstDay=1,lastDay=365,
-        startHour=0,stopHour=24,
-        url="http://fc.itis.cs.msu.ru/",
+        startHour=0,stopHour=23,
+        url="http://fetchclimate2.cloudapp.net/",
         dataSets="ANY",
-        reproduceFor="NOW"
+        reproduceFor="NOW",
+        verbose=F
 ) {
         #envVar is string
         #lat,lon are vectors with the same length (can be length of 1).
@@ -324,9 +479,9 @@ fcTimeSeriesYearly<-function(
         
         years <- seq(from=firstYear,to=lastYear+1,by=1)
         days <- c(firstDay,lastDay+1)
-        hours <- c(startHour,stopHour)
+        hours <- c(startHour,stopHour+1)
         
-        resultMatrix <-internal_fc.TimeSeries(variable,latitude,longitude,years,days,hours,url,dataSets,reproduceFor)
+        resultMatrix <-internal_fc.TimeSeries(variable,latitude,longitude,years,days,hours,url,dataSets,reproduceFor,verbose)
         
         resultMatrix$years <- years[1:(length(years)-1)]
         
@@ -343,10 +498,12 @@ fcTimeSeriesDaily<-function(
         latitude,longitude,
         firstDay=1,lastDay=365,
         firstYear=1961,lastYear=1990,
-        startHour=0,stopHour=24,
-        url="http://fc.itis.cs.msu.ru/",
+        startHour=0,stopHour=23,
+        url="http://fetchclimate2.cloudapp.net/",
         dataSets="ANY",
-        reproduceFor="NOW") {
+        reproduceFor="NOW",
+        verbose=F
+        ) {
         #envVar is string
         #lat,lon are vectors with the same length (can be length of 1).  
         #firstDay,lastDay are scalars
@@ -356,9 +513,9 @@ fcTimeSeriesDaily<-function(
         
         years <- c(firstYear,lastYear+1)
         days <- seq(from=firstDay,to=lastDay+1,by=1)
-        hours <- c(startHour,stopHour)
+        hours <- c(startHour,stopHour+1)
         
-        resultMatrix <-internal_fc.TimeSeries(variable,latitude,longitude,years,days,hours,url,dataSets,reproduceFor)
+        resultMatrix <-internal_fc.TimeSeries(variable,latitude,longitude,years,days,hours,url,dataSets,reproduceFor,verbose)
         
         resultMatrix$days <- days[1:(length(days)-1)]
         
@@ -376,9 +533,10 @@ fcTimeSeriesHourly<-function(
         startHour,stopHour,
         firstYear=1961,lastYear=1990,
         firstDay=1,lastDay=365,
-        url="http://fc.itis.cs.msu.ru/",
+        url="http://fetchclimate2.cloudapp.net/",
         dataSets="ANY",
-        reproduceFor="NOW") {
+        reproduceFor="NOW",
+        verbose=F) {
         #envVar is string
         #lat,lon are vectors with the same length (can be length of 1).
         #startHour,stopHour are scalars
@@ -388,11 +546,11 @@ fcTimeSeriesHourly<-function(
         
         years <- c(firstYear,lastYear+1)
         days <- c(firstDay,lastDay+1)
-        hours <- seq(from=startHour,to=stopHour)
+        hours <- seq(from=startHour,to=stopHour+1, by = 1)
         
-        resultMatrix <-internal_fc.TimeSeries(variable,latitude,longitude,years,days,hours,url,dataSets,reproduceFor)
+        resultMatrix <-internal_fc.TimeSeries(variable,latitude,longitude,years,days,hours,url,dataSets,reproduceFor,verbose)
         
-        resultMatrix$hours <- hours[1:(length(hours))]
+        resultMatrix$hours <- hours[1:(length(hours)-1)]
         
         return(resultMatrix)
 }
@@ -408,6 +566,7 @@ fcTimeSeriesHourly<-function(
 #' @param longitudeFrom A numeric scalar. The lower longitude bound of the spatial grid
 #' @param longitudeTo A numeric scalar. The upper longitude bound of the spatial grid
 #' @param longitudeBy A numeric scalar. The step of the grid along longitudes.
+#' @param verbose A logical scalar. If set to TRUE the method outputs its actions verbosely
 #' @inheritParams TimeSeries
 #' @return Type: SpatialPixelsDataFrame (from sp package)
 #' Contains a grid definition the following fields: values, sd, provenance
@@ -430,9 +589,10 @@ fcGrid <- function(
         firstYear=1961,lastYear=1990,
         firstDay=1,lastDay=365,
         startHour=0,stopHour=24,
-        url="http://fc.itis.cs.msu.ru/",
+        url="http://fetchclimate2.cloudapp.net/",
         dataSets="ANY",
-        reproduceFor="NOW") {
+        reproduceFor="NOW",
+        verbose=F) {
         
         lats <- seq(from=latitudeFrom,to=latitudeTo,by=latitudeBy)
         lons <- seq(from=longitudeFrom,to=longitudeTo,by=longitudeBy)
@@ -444,9 +604,9 @@ fcGrid <- function(
         requestBody <- internal_fc.formRequestBody(variable,
                                                    lats,lons,
                                                    years,days,hours,
-                                                   "PointGrid",dataSets,reproduceFor)
+                                                   "PointGrid",dataSets,reproduceFor,verbose)
         requestProvenance <- length(dataSets)>1 || dataSets=="ANY"  
-        response <- internal_fc.fetchCore(requestBody,url,requestProvenance)
+        response <- internal_fc.fetchCore(requestBody,url,requestProvenance,verbose)
         conf <- internal_fc.getConfiguration(url,reproduceFor)
         explicitDs <- c()
         if(requestProvenance){
